@@ -19,37 +19,32 @@ def generate_parse(print_function):
         (statements,) = pack
         return [x for x in statements if x] if statements else []
 
-
     @pg.production("statements : statements statement")
     def statements_statement(pack):
         (statements, statement) = pack
         return statements + statement
 
-
     @pg.production("statements : statement SEMICOLON")
     def statement_semicolon(pack):
         (statement, semicolon) = pack
         return statement +\
-                [{
-                  "type": "semicolon",
-                  "first_formatting": semicolon.hidden_tokens_before,
-                  "second_formatting": semicolon.hidden_tokens_after,
-                  "value": ";"
-                 }]
-
+            [{
+                "type": "semicolon",
+                "first_formatting": semicolon.hidden_tokens_before,
+                "second_formatting": semicolon.hidden_tokens_after,
+                "value": ";"
+            }]
 
     @pg.production("statements : statement")
     def statement(pack):
         (statement,) = pack
         return statement
 
-
     @pg.production("statement : endl")
     def statement_endl(pack):
         (endl,) = pack
         return endl
 
-
     @pg.production("endl : ENDL")
     def endl(pack):
         (endl,) = pack
@@ -64,13 +59,11 @@ def generate_parse(print_function):
             "indent": indent,
         }] + endl.hidden_tokens_after
 
-
     @pg.production("left_parenthesis : LEFT_PARENTHESIS")
     def left_parenthesis(pack):
         (lp,) = pack
         return lp
 
-
     @pg.production("endl : COMMENT ENDL")
     def comment(pack):
         (comment_, endl) = pack
@@ -85,12 +78,10 @@ def generate_parse(print_function):
             "value": endl.value
         }]
 
-
     @pg.production("statement : ENDMARKER")
     def end(_):
         return [None]
 
-
     @pg.production("statement : simple_stmt")
     @pg.production("statement : compound_stmt")
     def statement_simple_statement(pack):
@@ -102,31 +93,28 @@ def generate_parse(print_function):
         (small_stmt, semicolon, endl) = pack
         return [small_stmt,
                 {
-                 "type": "semicolon",
-                 "value": ";",
-                 "first_formatting": semicolon.hidden_tokens_before,
-                 "second_formatting": semicolon.hidden_tokens_after
+                    "type": "semicolon",
+                    "value": ";",
+                    "first_formatting": semicolon.hidden_tokens_before,
+                    "second_formatting": semicolon.hidden_tokens_after
                 }] + endl
 
-
     @pg.production("simple_stmt : small_stmt endl")
     def simple_stmt(pack):
         (small_stmt, endl) = pack
         return [small_stmt] + endl
 
-
     @pg.production("simple_stmt : small_stmt SEMICOLON simple_stmt")
     def simple_stmt_semicolon(pack):
         (small_stmt, semicolon, simple_stmt) = pack
         return [small_stmt,
                 {
-                 "type": "semicolon",
-                 "value": ";",
-                 "first_formatting": semicolon.hidden_tokens_before,
-                 "second_formatting": semicolon.hidden_tokens_after
+                    "type": "semicolon",
+                    "value": ";",
+                    "first_formatting": semicolon.hidden_tokens_before,
+                    "second_formatting": semicolon.hidden_tokens_after
                 }] + simple_stmt
 
-
     @pg.production("small_stmt : flow_stmt")
     @pg.production("small_stmt : del_stmt")
     @pg.production("small_stmt : pass_stmt")
@@ -147,7 +135,6 @@ def generate_parse(print_function):
         (statement,) = pack
         return statement
 
-
     @pg.production("async_maybe : ")
     def async_maybe(pack):
         return {
@@ -155,7 +142,6 @@ def generate_parse(print_function):
             "formatting": [],
         }
 
-
     @pg.production("async_maybe : NAME")
     @pg.production("async : NAME")
     def async_without_space(pack):
@@ -166,7 +152,6 @@ def generate_parse(print_function):
             "formatting": [],
         }
 
-
     @pg.production("async_maybe : NAME SPACE")
     @pg.production("async : NAME SPACE")
     def async_(pack):
@@ -177,7 +162,6 @@ def generate_parse(print_function):
             "formatting": [{'type': 'space', 'value': space.value}],
         }
 
-
     @pg.production("async_stmt : async with_stmt")
     @pg.production("async_stmt : async for_stmt")
     def async_stmt(pack):
@@ -190,14 +174,12 @@ def generate_parse(print_function):
         statement[0]["async_formatting"] += async_["formatting"]
         return statement
 
-
     if not print_function:
         @pg.production("small_stmt : print_stmt")
         def print_statement(pack):
             (statement,) = pack
             return statement
 
-
     @pg.production("small_stmt : expr_stmt")
     @pg.production("expr_stmt : testlist")
     @pg.production("testlist : test")
@@ -220,7 +202,6 @@ def generate_parse(print_function):
         (level,) = pack
         return level
 
-
     @pg.production("with_stmt : WITH with_items COLON suite")
     def with_stmt(pack):
         (with_, with_items, colon, suite) = pack
@@ -235,19 +216,16 @@ def generate_parse(print_function):
             "contexts": with_items
         }]
 
-
     @pg.production("with_items : with_items comma with_item")
     def with_items_with_item(pack):
         (with_items, comma, with_item,) = pack
         return with_items + [comma, with_item]
 
-
     @pg.production("with_items : with_item")
     def with_items(pack):
         (with_item,) = pack
         return [with_item]
 
-
     @pg.production("with_item : test")
     def with_item(pack):
         (test,) = pack
@@ -259,7 +237,6 @@ def generate_parse(print_function):
             "value": test
         }
 
-
     @pg.production("with_item : test AS expr")
     def with_item_as(pack):
         (test, as_, expr) = pack
@@ -271,7 +248,6 @@ def generate_parse(print_function):
             "value": test
         }
 
-
     @pg.production("classdef : CLASS NAME COLON suite")
     def class_stmt(pack,):
         (class_, name, colon, suite) = pack
@@ -290,7 +266,6 @@ def generate_parse(print_function):
             "value": suite,
         }]
 
-
     @pg.production("classdef : CLASS NAME LEFT_PARENTHESIS RIGHT_PARENTHESIS COLON suite")
     def class_stmt_parenthesis(pack,):
         (class_, name, left_parenthesis, right_parenthesis, colon, suite) = pack
@@ -309,7 +284,6 @@ def generate_parse(print_function):
             "value": suite,
         }]
 
-
     @pg.production("classdef : CLASS NAME LEFT_PARENTHESIS testlist RIGHT_PARENTHESIS COLON suite")
     def class_stmt_inherit(pack,):
         (class_, name, left_parenthesis, testlist, right_parenthesis, colon, suite) = pack
@@ -328,7 +302,6 @@ def generate_parse(print_function):
             "value": suite,
         }]
 
-
     @pg.production("decorated : decorators funcdef")
     @pg.production("decorated : decorators classdef")
     def decorated(pack):
@@ -336,19 +309,16 @@ def generate_parse(print_function):
         funcdef[0]["decorators"] = decorators
         return funcdef
 
-
     @pg.production("decorators : decorators decorator")
     def decorators_decorator(pack):
         (decorators, decorator,) = pack
         return decorators + decorator
 
-
     @pg.production("decorators : decorator")
     def decorators(pack):
         (decorator,) = pack
         return decorator
 
-
     # TODO tests
     @pg.production("decorator : endl")
     def decorator_endl(pack):
@@ -357,7 +327,6 @@ def generate_parse(print_function):
         (endl,) = pack
         return endl
 
-
     @pg.production("decorator : AT dotted_name endl")
     def decorator(pack):
         (at, dotted_name, endl) = pack
@@ -371,7 +340,6 @@ def generate_parse(print_function):
             "formatting": at.hidden_tokens_after,
         }] + endl
 
-
     @pg.production("decorator : AT dotted_name LEFT_PARENTHESIS RIGHT_PARENTHESIS endl")
     def decorator_empty_call(pack):
         (at, dotted_name, left_parenthesis, right_parenthesis, endl) = pack
@@ -392,7 +360,6 @@ def generate_parse(print_function):
             "formatting": at.hidden_tokens_after,
         }] + endl
 
-
     @pg.production("decorator : AT dotted_name LEFT_PARENTHESIS argslist RIGHT_PARENTHESIS endl")
     def decorator_call(pack):
         (at, dotted_name, left_parenthesis, argslist, right_parenthesis, endl) = pack
@@ -413,9 +380,9 @@ def generate_parse(print_function):
             "formatting": at.hidden_tokens_after,
         }] + endl
 
-    @pg.production("funcdef : async_maybe DEF NAME LEFT_PARENTHESIS parameters RIGHT_PARENTHESIS COLON suite")
+    @pg.production("funcdef : async_maybe DEF NAME LEFT_PARENTHESIS typed_parameters RIGHT_PARENTHESIS COLON suite")
     def function_definition(pack):
-        (async_maybe, def_, name, left_parenthesis, parameters, right_parenthesis, colon, suite) = pack
+        (async_maybe, def_, name, left_parenthesis, typed_parameters, right_parenthesis, colon, suite) = pack
 
         if async_maybe["async"] and async_maybe["value"] != "async":
             raise ParsingError("The only possible keyword before a 'def' is 'async', not '%s'" % async_maybe["value"])
@@ -432,23 +399,26 @@ def generate_parse(print_function):
             "fourth_formatting": right_parenthesis.hidden_tokens_before,
             "fifth_formatting": colon.hidden_tokens_before,
             "sixth_formatting": colon.hidden_tokens_after,
-            "arguments": parameters,
+            "arguments": typed_parameters,
             "value": suite,
         }]
 
     @pg.production("argslist : argslist argument")
+    @pg.production("typed_parameters : typed_parameters typed_parameter")
     @pg.production("parameters : parameters parameter")
     def parameters_parameters_parameter(pack,):
         (parameters, parameter,) = pack
         return parameters + parameter
 
     @pg.production("argslist : argument")
+    @pg.production("typed_parameters : typed_parameter")
     @pg.production("parameters : parameter")
     def parameters_parameter(pack,):
         (parameter,) = pack
         return parameter
 
     @pg.production("argument :")
+    @pg.production("typed_parameter : ")
     @pg.production("parameter : ")
     def parameter_empty(p):
         return []
@@ -461,6 +431,26 @@ def generate_parse(print_function):
             "value": name_.value,
         }
 
+    @pg.production("typed_name : NAME COLON test")
+    def typed_name_with_type(pack):
+        (name_, colon, test) = pack
+        return {
+            "type": "typed_name",
+            "first_formatting": colon.hidden_tokens_before if colon else [],
+            "second_formatting": colon.hidden_tokens_after if colon else [],
+            "value": name_.value,
+            "annotation": test
+        }
+
+    @pg.production("typed_name : NAME")
+    def typed_name_no_type(pack):
+        (name_,) = pack
+        return {
+            "type": "name",
+            "value": name_.value,
+        }
+
+    @pg.production("typed_parameter : LEFT_PARENTHESIS name RIGHT_PARENTHESIS maybe_test")
     @pg.production("parameter : LEFT_PARENTHESIS name RIGHT_PARENTHESIS maybe_test")
     def parameter_fpdef(pack):
         (left_parenthesis, name, right_parenthesis, (equal, test)) = pack
@@ -479,7 +469,7 @@ def generate_parse(print_function):
             }
         }]
 
-
+    @pg.production("typed_parameter : LEFT_PARENTHESIS fplist RIGHT_PARENTHESIS maybe_test")
     @pg.production("parameter : LEFT_PARENTHESIS fplist RIGHT_PARENTHESIS maybe_test")
     def parameter_fplist(pack):
         (left_parenthesis, fplist, right_parenthesis, (equal, test)) = pack
@@ -499,7 +489,6 @@ def generate_parse(print_function):
             },
         }]
 
-
     @pg.production("fplist : fplist parameter")
     def fplist_recur(pack):
         (fplist, name) = pack
@@ -507,7 +496,6 @@ def generate_parse(print_function):
             name = [name[0]["target"]]
         return fplist + name
 
-
     @pg.production("fplist : parameter comma")
     def fplist(pack):
         (name, comma) = pack
@@ -515,7 +503,6 @@ def generate_parse(print_function):
             name = [name[0]["target"]]
         return name + [comma]
 
-
     # really strange that left part of argument grammar can be a test
     # I guess it's yet another legacy mistake
     # python give me 'SyntaxError: keyword can't be an expression' when I try to
@@ -531,6 +518,7 @@ def generate_parse(print_function):
             "target": name if equal else {}
         }]
 
+    @pg.production("typed_parameter : typed_name maybe_test")
     @pg.production("parameter : name maybe_test")
     def parameter_with_default(pack):
         (name, (equal, test)) = pack
@@ -577,7 +565,38 @@ def generate_parse(print_function):
             "value": test,
         }]
 
+    @pg.production("typed_parameter : STAR NAME COLON test")
+    def typed_parameter_star(pack):
+        (star, name, colon, test) = pack
+        return [{
+            "type": "list_argument",
+            "formatting": star.hidden_tokens_after,
+            "value": {
+                "type": "typed_name",
+                "first_formatting": colon.hidden_tokens_before if colon else [],
+                "second_formatting": colon.hidden_tokens_after if colon else [],
+                "value": name.value,
+                "annotation": test
+            }
+        }]
+
+    @pg.production("typed_parameter : DOUBLE_STAR NAME COLON test")
+    def typed_parameter_double_star(pack):
+        (double_star, name, colon, test) = pack
+        return [{
+            "type": "dict_argument",
+            "formatting": double_star.hidden_tokens_after,
+            "value": {
+                "type": "typed_name",
+                "first_formatting": colon.hidden_tokens_before if colon else [],
+                "second_formatting": colon.hidden_tokens_after if colon else [],
+                "value": name.value,
+                "annotation": test
+            }
+        }]
+
     # TODO refactor those 2 to standardize with argument_star and argument_star_star
+    @pg.production("typed_parameter : STAR NAME")
     @pg.production("parameter : STAR NAME")
     def parameter_star(pack):
         (star, name,) = pack
@@ -591,6 +610,7 @@ def generate_parse(print_function):
         }]
 
     # TODO refactor those 2 to standardize with argument_star and argument_star_star
+    @pg.production("typed_parameter : STAR")
     @pg.production("parameter : STAR")
     def parameter_star_only(pack):
         (star, ) = pack
@@ -599,6 +619,7 @@ def generate_parse(print_function):
             "formatting": star.hidden_tokens_after,
         }]
 
+    @pg.production("typed_parameter : DOUBLE_STAR NAME")
     @pg.production("parameter : DOUBLE_STAR NAME")
     def parameter_star_star(pack):
         (double_star, name,) = pack
@@ -612,6 +633,7 @@ def generate_parse(print_function):
         }]
 
     @pg.production("argument : comma")
+    @pg.production("typed_parameter : comma")
     @pg.production("parameter : comma")
     def parameter_comma(pack):
         (comma,) = pack
@@ -622,13 +644,11 @@ def generate_parse(print_function):
         (simple_stmt,) = pack
         return simple_stmt
 
-
     @pg.production("suite : endls INDENT statements DEDENT")
     def suite_indent(pack):
         (endls, indent, statements, dedent,) = pack
         return endls + statements
 
-
     @pg.production("endls : endls endl")
     @pg.production("endls : endl")
     def endls(p):
@@ -636,7 +656,6 @@ def generate_parse(print_function):
             return p[0]
         return p[0] + p[1]
 
-
     include_imports(pg)
     include_control_structures(pg)
     include_primivites(pg, print_function)
@@ -688,13 +707,11 @@ def generate_parse(print_function):
         (int_,) = pack
         return create_node_from_token(int_, section="number")
 
-
     @pg.production("atom : name")
     def atom_name(pack):
         (name,) = pack
         return name
 
-
     @pg.production("atom : strings")
     def strings(pack):
         (string_chain,) = pack
@@ -733,7 +750,6 @@ def generate_parse(print_function):
             "second_formatting": string_.hidden_tokens_after,
         }]
 
-
     @pg.production("comma : COMMA")
     def comma(pack):
         (comma,) = pack
